@@ -29,11 +29,27 @@
 #include <freerdp/freerdp.h>
 #include <freerdp/channels/drdynvc.h>
 #include <freerdp/utils/drdynvc.h>
+#include <freerdp/codec/zgfx.h>
 
 #include "drdynvc_main.h"
 
 #define TAG CHANNELS_TAG("drdynvc.client")
 
+static const char* channel_state2str(DVC_CHANNEL_STATE state)
+{
+	switch (state)
+	{
+		case DVC_CHANNEL_INIT:
+			return "DVC_CHANNEL_INIT";
+		case DVC_CHANNEL_RUNNING:
+			return "DVC_CHANNEL_RUNNING";
+		case DVC_CHANNEL_CLOSED:
+			return "DVC_CHANNEL_CLOSED";
+		default:
+			return "DVC_CHANNEL_UNKNOWN";
+	}
+}
+
 static void dvcman_channel_free(DVCMAN_CHANNEL* channel);
 static UINT dvcman_channel_close(DVCMAN_CHANNEL* channel, BOOL perRequest, BOOL fromHashTableFn);
 static void dvcman_free(drdynvcPlugin* drdynvc, IWTSVirtualChannelManager* pChannelMgr);
@@ -390,6 +406,7 @@ static void dvcman_channel_free(DVCMAN_CHANNEL* channel)
 	if (channel->dvc_data)
 		Stream_Release(channel->dvc_data);
 
+	zgfx_context_free(channel->decompressor);
 	DeleteCriticalSection(&(channel->lock));
 	free(channel->channel_name);
 	free(channel);
@@ -433,12 +450,9 @@ static void check_open_close_receive(DVCMAN_CHANNEL* channel)
 	const UINT32 id = channel->channel_id;
 
 	WINPR_ASSERT(cb);
-	if (cb->OnOpen || cb->OnClose)
-	{
-		if (!cb->OnOpen || !cb->OnClose)
-			WLog_WARN(TAG, "{%s:%" PRIu32 "} OnOpen=%p, OnClose=%p", name, id, cb->OnOpen,
-			          cb->OnClose);
-	}
+	if (!cb->OnOpen || !cb->OnClose || !cb->OnDataReceived)
+		WLog_VRB(TAG, "{%s:%" PRIu32 "} OnOpen=%p, OnClose=%p, OnDataReceived=%p", name, id,
+		         cb->OnOpen, cb->OnClose, cb->OnDataReceived);
 }
 
 static UINT dvcman_call_on_receive(DVCMAN_CHANNEL* channel, wStream* data)
@@ -477,12 +491,17 @@ static UINT dvcman_channel_close(DVCMAN_CHANNEL* channel, BOOL perRequest, BOOL
 				error = dvcchannel_send_close(channel);
 				if (error != CHANNEL_RC_OK)
 				{
-					const char* msg = "error when sending close confirm for '%s'";
 					if (perRequest)
-						msg = "error when sending closeRequest for '%s'";
-
-					WLog_Print(drdynvc->log, WLOG_DEBUG, msg, channel->channel_name);
+						WLog_Print(drdynvc->log, WLOG_DEBUG,
+						           "error when sending closeRequest for '%s'",
+						           channel->channel_name);
+					else
+						WLog_Print(drdynvc->log, WLOG_DEBUG,
+						           "error when sending close confirm for '%s'",
+						           channel->channel_name);
 				}
+				WLog_Print(drdynvc->log, WLOG_DEBUG, "listener %s destroyed channel %" PRIu32 "",
+				           channel->channel_name, channel->channel_id);
 			}
 
 			channel->state = DVC_CHANNEL_CLOSED;
@@ -535,10 +554,13 @@ static DVCMAN_CHANNEL* dvcman_channel_new(WINPR_ATTR_UNUSED drdynvcPlugin* drdyn
 	channel->refCounter = 1;
 	channel->state = DVC_CHANNEL_INIT;
 	channel->channel_name = _strdup(ChannelName);
-
 	if (!channel->channel_name)
 		goto fail;
 
+	channel->decompressor = zgfx_context_new(FALSE);
+	if (!channel->decompressor)
+		goto fail;
+
 	if (!InitializeCriticalSectionEx(&(channel->lock), 0, 0))
 		goto fail;
 
@@ -691,10 +713,12 @@ static DVCMAN_CHANNEL* dvcman_create_channel(drdynvcPlugin* drdynvc,
 			case DVC_CHANNEL_CLOSED:
 			case DVC_CHANNEL_INIT:
 			default:
-				WLog_Print(drdynvc->log, WLOG_ERROR, "not expecting a createChannel from state %d",
-				           channel->state);
+			{
+				WLog_Print(drdynvc->log, WLOG_ERROR, "not expecting a createChannel from state %s",
+				           channel_state2str(channel->state));
 				*res = CHANNEL_RC_INITIALIZATION_ERROR;
 				goto out;
+			}
 		}
 	}
 	else
@@ -945,7 +969,6 @@ static UINT drdynvc_write_data(drdynvcPlugin* drdynvc, UINT32 ChannelId, const B
 	size_t pos = 0;
 	UINT8 cbChId = 0;
 	UINT8 cbLen = 0;
-	unsigned long chunkLength = 0;
 	UINT status = CHANNEL_RC_BAD_INIT_HANDLE;
 	DVCMAN* dvcman = NULL;
 
@@ -993,10 +1016,16 @@ static UINT drdynvc_write_data(drdynvcPlugin* drdynvc, UINT32 ChannelId, const B
 		const INT32 pdu = (DATA_FIRST_PDU << 4) | cbChId | (cbLen << 2);
 		Stream_Write_UINT8(data_out, WINPR_ASSERTING_INT_CAST(UINT8, pdu));
 		Stream_SetPosition(data_out, pos);
-		chunkLength = CHANNEL_CHUNK_LENGTH - pos;
-		Stream_Write(data_out, data, chunkLength);
-		data += chunkLength;
-		dataSize -= chunkLength;
+
+		{
+			WINPR_ASSERT(pos <= CHANNEL_CHUNK_LENGTH);
+			const uint32_t chunkLength =
+			    CHANNEL_CHUNK_LENGTH - WINPR_ASSERTING_INT_CAST(uint32_t, pos);
+			Stream_Write(data_out, data, chunkLength);
+
+			data += chunkLength;
+			dataSize -= chunkLength;
+		}
 		status = drdynvc_send(drdynvc, data_out);
 
 		while (status == CHANNEL_RC_OK && dataSize > 0)
@@ -1015,10 +1044,12 @@ static UINT drdynvc_write_data(drdynvcPlugin* drdynvc, UINT32 ChannelId, const B
 			Stream_SetPosition(data_out, 0);
 			Stream_Write_UINT8(data_out, (DATA_PDU << 4) | cbChId);
 			Stream_SetPosition(data_out, pos);
-			chunkLength = dataSize;
 
+			uint32_t chunkLength = dataSize;
+
+			WINPR_ASSERT(pos <= CHANNEL_CHUNK_LENGTH);
 			if (chunkLength > CHANNEL_CHUNK_LENGTH - pos)
-				chunkLength = CHANNEL_CHUNK_LENGTH - pos;
+				chunkLength = CHANNEL_CHUNK_LENGTH - WINPR_ASSERTING_INT_CAST(uint32_t, pos);
 
 			Stream_Write(data_out, data, chunkLength);
 			data += chunkLength;
@@ -1268,25 +1299,20 @@ static UINT drdynvc_process_create_request(drdynvcPlugin* drdynvc, UINT8 Sp, UIN
  * @return 0 on success, otherwise a Win32 error code
  */
 static UINT drdynvc_process_data_first(drdynvcPlugin* drdynvc, int Sp, int cbChId, wStream* s,
-                                       UINT32 ThreadingFlags)
+                                       BOOL compressed, UINT32 ThreadingFlags)
 {
-	UINT status = CHANNEL_RC_OK;
-	UINT32 Length = 0;
-	UINT32 ChannelId = 0;
-	DVCMAN_CHANNEL* channel = NULL;
-
 	WINPR_ASSERT(drdynvc);
 	if (!Stream_CheckAndLogRequiredLength(
 	        TAG, s, drdynvc_cblen_to_bytes(cbChId) + drdynvc_cblen_to_bytes(Sp)))
 		return ERROR_INVALID_DATA;
 
-	ChannelId = drdynvc_read_variable_uint(s, cbChId);
-	Length = drdynvc_read_variable_uint(s, Sp);
+	UINT32 ChannelId = drdynvc_read_variable_uint(s, cbChId);
+	UINT32 Length = drdynvc_read_variable_uint(s, Sp);
 	WLog_Print(drdynvc->log, WLOG_TRACE,
 	           "process_data_first: Sp=%d cbChId=%d, ChannelId=%" PRIu32 " Length=%" PRIu32 "", Sp,
 	           cbChId, ChannelId, Length);
 
-	channel = dvcman_get_channel_by_id(drdynvc->channel_mgr, ChannelId, TRUE);
+	DVCMAN_CHANNEL* channel = dvcman_get_channel_by_id(drdynvc->channel_mgr, ChannelId, TRUE);
 	if (!channel)
 	{
 		/**
@@ -1298,9 +1324,36 @@ static UINT drdynvc_process_data_first(drdynvcPlugin* drdynvc, int Sp, int cbChI
 		return CHANNEL_RC_OK;
 	}
 
+	UINT status = CHANNEL_RC_OK;
+	BOOL shouldFree = FALSE;
 	if (channel->state != DVC_CHANNEL_RUNNING)
 		goto out;
 
+	if (compressed)
+	{
+		BYTE* data = NULL;
+		UINT32 dataSize = 0;
+		if (zgfx_decompress(channel->decompressor, Stream_Pointer(s),
+		                    WINPR_ASSERTING_INT_CAST(UINT32, Stream_GetRemainingLength(s)), &data,
+		                    &dataSize, 0) < 0)
+		{
+			status = ERROR_INVALID_DATA;
+			WLog_Print(drdynvc->log, WLOG_ERROR, "error de-compressing first packet");
+			goto out;
+		}
+
+		s = Stream_New(data, dataSize);
+		if (!s)
+		{
+			status = CHANNEL_RC_NO_MEMORY;
+			WLog_Print(drdynvc->log, WLOG_ERROR, "error allocating new Stream(len=%" PRIu32 ")",
+			           dataSize);
+			free(data);
+			goto out;
+		}
+		shouldFree = TRUE;
+	}
+
 	status = dvcman_receive_channel_data_first(channel, Length);
 
 	if (status == CHANNEL_RC_OK)
@@ -1310,6 +1363,8 @@ static UINT drdynvc_process_data_first(drdynvcPlugin* drdynvc, int Sp, int cbChI
 		status = dvcman_channel_close(channel, FALSE, FALSE);
 
 out:
+	if (shouldFree)
+		Stream_Free(s, TRUE);
 	dvcman_channel_unref(channel);
 	return status;
 }
@@ -1320,21 +1375,17 @@ out:
  * @return 0 on success, otherwise a Win32 error code
  */
 static UINT drdynvc_process_data(drdynvcPlugin* drdynvc, int Sp, int cbChId, wStream* s,
-                                 UINT32 ThreadingFlags)
+                                 BOOL compressed, UINT32 ThreadingFlags)
 {
-	UINT32 ChannelId = 0;
-	DVCMAN_CHANNEL* channel = NULL;
-	UINT status = CHANNEL_RC_OK;
-
 	WINPR_ASSERT(drdynvc);
 	if (!Stream_CheckAndLogRequiredLength(TAG, s, drdynvc_cblen_to_bytes(cbChId)))
 		return ERROR_INVALID_DATA;
 
-	ChannelId = drdynvc_read_variable_uint(s, cbChId);
+	UINT32 ChannelId = drdynvc_read_variable_uint(s, cbChId);
 	WLog_Print(drdynvc->log, WLOG_TRACE, "process_data: Sp=%d cbChId=%d, ChannelId=%" PRIu32 "", Sp,
 	           cbChId, ChannelId);
 
-	channel = dvcman_get_channel_by_id(drdynvc->channel_mgr, ChannelId, TRUE);
+	DVCMAN_CHANNEL* channel = dvcman_get_channel_by_id(drdynvc->channel_mgr, ChannelId, TRUE);
 	if (!channel)
 	{
 		/**
@@ -1346,14 +1397,44 @@ static UINT drdynvc_process_data(drdynvcPlugin* drdynvc, int Sp, int cbChId, wSt
 		return CHANNEL_RC_OK;
 	}
 
+	BOOL shouldFree = FALSE;
+	UINT status = CHANNEL_RC_OK;
 	if (channel->state != DVC_CHANNEL_RUNNING)
 		goto out;
 
+	if (compressed)
+	{
+		BYTE* data = NULL;
+		UINT32 dataSize = 0;
+
+		if (zgfx_decompress(channel->decompressor, Stream_Pointer(s),
+		                    WINPR_ASSERTING_INT_CAST(UINT32, Stream_GetRemainingLength(s)), &data,
+		                    &dataSize, 0) < 0)
+		{
+			status = ERROR_INVALID_DATA;
+			WLog_Print(drdynvc->log, WLOG_ERROR, "error de-compressing data packet");
+			goto out;
+		}
+
+		s = Stream_New(data, dataSize);
+		if (!s)
+		{
+			status = CHANNEL_RC_NO_MEMORY;
+			WLog_Print(drdynvc->log, WLOG_ERROR, "error allocating new Stream(len=%" PRIu32 ")",
+			           dataSize);
+			free(data);
+			goto out;
+		}
+		shouldFree = TRUE;
+	}
+
 	status = dvcman_receive_channel_data(channel, s, ThreadingFlags);
 	if (status != CHANNEL_RC_OK)
 		status = dvcman_channel_close(channel, FALSE, FALSE);
 
 out:
+	if (shouldFree)
+		Stream_Free(s, TRUE);
 	dvcman_channel_unref(channel);
 	return status;
 }
@@ -1397,13 +1478,11 @@ static UINT drdynvc_process_close_request(drdynvcPlugin* drdynvc, int Sp, int cb
  */
 static UINT drdynvc_order_recv(drdynvcPlugin* drdynvc, wStream* s, UINT32 ThreadingFlags)
 {
-	UINT8 value = 0;
-
 	WINPR_ASSERT(drdynvc);
 	if (!Stream_CheckAndLogRequiredLength(TAG, s, 1))
 		return ERROR_INVALID_DATA;
 
-	Stream_Read_UINT8(s, value);
+	UINT8 value = Stream_Get_UINT8(s);
 	const UINT8 Cmd = (value & 0xf0) >> 4;
 	const UINT8 Sp = (value & 0x0c) >> 2;
 	const UINT8 cbChId = (value & 0x03) >> 0;
@@ -1419,14 +1498,23 @@ static UINT drdynvc_order_recv(drdynvcPlugin* drdynvc, wStream* s, UINT32 Thread
 			return drdynvc_process_create_request(drdynvc, Sp, cbChId, s);
 
 		case DATA_FIRST_PDU:
-			return drdynvc_process_data_first(drdynvc, Sp, cbChId, s, ThreadingFlags);
+		case DATA_FIRST_COMPRESSED_PDU:
+			return drdynvc_process_data_first(drdynvc, Sp, cbChId, s,
+			                                  (Cmd == DATA_FIRST_COMPRESSED_PDU), ThreadingFlags);
 
 		case DATA_PDU:
-			return drdynvc_process_data(drdynvc, Sp, cbChId, s, ThreadingFlags);
+		case DATA_COMPRESSED_PDU:
+			return drdynvc_process_data(drdynvc, Sp, cbChId, s, (Cmd == DATA_COMPRESSED_PDU),
+			                            ThreadingFlags);
 
 		case CLOSE_REQUEST_PDU:
 			return drdynvc_process_close_request(drdynvc, Sp, cbChId, s);
 
+		case SOFT_SYNC_RESPONSE_PDU:
+			WLog_Print(drdynvc->log, WLOG_ERROR,
+			           "not expecting a SOFT_SYNC_RESPONSE_PDU as a client");
+			return ERROR_INTERNAL_ERROR;
+
 		default:
 			WLog_Print(drdynvc->log, WLOG_ERROR, "unknown drdynvc cmd 0x%x", Cmd);
 			return ERROR_INTERNAL_ERROR;
@@ -1506,6 +1594,7 @@ static UINT drdynvc_virtual_channel_event_data_received(drdynvcPlugin* drdynvc,
 			{
 				WLog_Print(drdynvc->log, WLOG_WARN,
 				           "drdynvc_order_recv failed with error %" PRIu32 "!", error);
+				return error;
 			}
 		}
 	}
