@@ -41,18 +41,42 @@
 #if defined(CHANNEL_AUDIN_SERVER)
 #include <freerdp/server/audin.h>
 #endif
+#if defined(CHANNEL_RDPSND_SERVER)
 #include <freerdp/server/rdpsnd.h>
+#endif
+#if defined(CHANNEL_CLIPRDR_SERVER)
 #include <freerdp/server/cliprdr.h>
+#endif
+#if defined(CHANNEL_ECHO_SERVER)
 #include <freerdp/server/echo.h>
+#endif
+#if defined(CHANNEL_RDPDR_SERVER)
 #include <freerdp/server/rdpdr.h>
+#endif
+#if defined(CHANNEL_RDPEI_SERVER)
 #include <freerdp/server/rdpei.h>
+#endif
+#if defined(CHANNEL_DRDYNVC_SERVER)
 #include <freerdp/server/drdynvc.h>
+#endif
+#if defined(CHANNEL_REMDESK_SERVER)
 #include <freerdp/server/remdesk.h>
+#endif
+#if defined(CHANNEL_ENCOMSP_SERVER)
 #include <freerdp/server/encomsp.h>
+#endif
+#if defined(CHANNEL_RAIL_SERVER)
 #include <freerdp/server/rail.h>
+#endif
+#if defined(CHANNEL_TELEMETRY_SERVER)
 #include <freerdp/server/telemetry.h>
+#endif
+#if defined(CHANNEL_RDPGFX_SERVER)
 #include <freerdp/server/rdpgfx.h>
+#endif
+#if defined(CHANNEL_DISP_SERVER)
 #include <freerdp/server/disp.h>
+#endif
 
 #if defined(CHANNEL_RDPEMSC_SERVER)
 #include <freerdp/server/rdpemsc.h>
@@ -82,18 +106,42 @@ void freerdp_channels_dummy(void)
 #if defined(CHANNEL_AUDIN_SERVER)
 	audin_server_context* audin = NULL;
 #endif
+#if defined(CHANNEL_RDPSND_SERVER)
 	RdpsndServerContext* rdpsnd = NULL;
+#endif
+#if defined(CHANNEL_CLIPRDR_SERVER)
 	CliprdrServerContext* cliprdr = NULL;
+#endif
+#if defined(CHANNEL_ECHO_SERVER)
 	echo_server_context* echo = NULL;
+#endif
+#if defined(CHANNEL_RDPDR_SERVER)
 	RdpdrServerContext* rdpdr = NULL;
+#endif
+#if defined(CHANNEL_DRDYNVC_SERVER)
 	DrdynvcServerContext* drdynvc = NULL;
+#endif
+#if defined(CHANNEL_RDPEI_SERVER)
 	RdpeiServerContext* rdpei = NULL;
+#endif
+#if defined(CHANNEL_REMDESK_SERVER)
 	RemdeskServerContext* remdesk = NULL;
+#endif
+#if defined(CHANNEL_ENCOMSP_SERVER)
 	EncomspServerContext* encomsp = NULL;
+#endif
+#if defined(CHANNEL_RAIL_SERVER)
 	RailServerContext* rail = NULL;
+#endif
+#if defined(CHANNEL_TELEMETRY_SERVER)
 	TelemetryServerContext* telemetry = NULL;
+#endif
+#if defined(CHANNEL_RDPGFX_SERVER)
 	RdpgfxServerContext* rdpgfx = NULL;
+#endif
+#if defined(CHANNEL_DISP_SERVER)
 	DispServerContext* disp = NULL;
+#endif
 #if defined(CHANNEL_RDPEMSC_SERVER)
 	MouseCursorServerContext* mouse_cursor = NULL;
 #endif /* CHANNEL_RDPEMSC_SERVER */
@@ -113,31 +161,54 @@ void freerdp_channels_dummy(void)
 #if defined(CHANNEL_AUDIN_SERVER)
 	audin_server_context_free(audin);
 #endif
+#if defined(CHANNEL_RDPSND_SERVER)
 	rdpsnd = rdpsnd_server_context_new(NULL);
 	rdpsnd_server_context_free(rdpsnd);
+#endif
+#if defined(CHANNEL_CLIPRDR_SERVER)
 	cliprdr = cliprdr_server_context_new(NULL);
 	cliprdr_server_context_free(cliprdr);
+#endif
+#if defined(CHANNEL_ECHO_SERVER)
 	echo = echo_server_context_new(NULL);
 	echo_server_context_free(echo);
+#endif
+#if defined(CHANNEL_RDPDR_SERVER)
 	rdpdr = rdpdr_server_context_new(NULL);
 	rdpdr_server_context_free(rdpdr);
+#endif
+#if defined(CHANNEL_DRDYNVC_SERVER)
 	drdynvc = drdynvc_server_context_new(NULL);
 	drdynvc_server_context_free(drdynvc);
+#endif
+#if defined(CHANNEL_RDPEI_SERVER)
 	rdpei = rdpei_server_context_new(NULL);
 	rdpei_server_context_free(rdpei);
+#endif
+#if defined(CHANNEL_REMDESK_SERVER)
 	remdesk = remdesk_server_context_new(NULL);
 	remdesk_server_context_free(remdesk);
+#endif
+#if defined(CHANNEL_ENCOMSP_SERVER)
 	encomsp = encomsp_server_context_new(NULL);
 	encomsp_server_context_free(encomsp);
+#endif
+#if defined(CHANNEL_RAIL_SERVER)
 	rail = rail_server_context_new(NULL);
 	rail_server_context_free(rail);
+#endif
+#if defined(CHANNEL_TELEMETRY_SERVER)
 	telemetry = telemetry_server_context_new(NULL);
 	telemetry_server_context_free(telemetry);
+#endif
+#if defined(CHANNEL_RDPGFX_SERVER)
 	rdpgfx = rdpgfx_server_context_new(NULL);
 	rdpgfx_server_context_free(rdpgfx);
+#endif
+#if defined(CHANNEL_DISP_SERVER)
 	disp = disp_server_context_new(NULL);
 	disp_server_context_free(disp);
-
+#endif
 #if defined(CHANNEL_RDPEMSC_SERVER)
 	mouse_cursor = mouse_cursor_server_context_new(NULL);
 	mouse_cursor_server_context_free(mouse_cursor);
