@@ -33,8 +33,6 @@
 #include "sdl_select.hpp"
 #include "sdl_selectlist.hpp"
 
-#define TAG CLIENT_TAG("SDL.dialogs")
-
 enum
 {
 	SHOW_DIALOG_ACCEPT_REJECT = 1,
@@ -55,7 +53,7 @@ static const char* type_str_for_flags(UINT32 flags)
 
 static BOOL sdl_wait_for_result(rdpContext* context, Uint32 type, SDL_Event* result)
 {
-	const SDL_Event empty = { 0 };
+	const SDL_Event empty = {};
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(result);
@@ -74,7 +72,7 @@ static BOOL sdl_wait_for_result(rdpContext* context, Uint32 type, SDL_Event* res
 static int sdl_show_dialog(rdpContext* context, const char* title, const char* message,
                            Sint32 flags)
 {
-	SDL_Event event = { 0 };
+	SDL_Event event = {};
 
 	if (!sdl_push_user_event(SDL_USEREVENT_SHOW_DIALOG, title, message, flags))
 		return 0;
@@ -88,7 +86,7 @@ static int sdl_show_dialog(rdpContext* context, const char* title, const char* m
 BOOL sdl_authenticate_ex(freerdp* instance, char** username, char** password, char** domain,
                          rdp_auth_reason reason)
 {
-	SDL_Event event = { 0 };
+	SDL_Event event = {};
 	BOOL res = FALSE;
 
 	SDLConnectionDialogHider hider(instance);
@@ -119,6 +117,7 @@ BOOL sdl_authenticate_ex(freerdp* instance, char** username, char** password, ch
 	size_t titlesize = 0;
 	winpr_asprintf(&title, &titlesize, "Credentials required for %s", target);
 
+	CStringPtr scope(title, free);
 	char* u = nullptr;
 	char* d = nullptr;
 	char* p = nullptr;
@@ -132,26 +131,22 @@ BOOL sdl_authenticate_ex(freerdp* instance, char** username, char** password, ch
 	p = *password;
 
 	if (!sdl_push_user_event(SDL_USEREVENT_AUTH_DIALOG, title, u, d, p, reason))
-		goto fail;
+		return res;
 
 	if (!sdl_wait_for_result(instance->context, SDL_USEREVENT_AUTH_RESULT, &event))
-		goto fail;
-	else
-	{
-		auto arg = reinterpret_cast<SDL_UserAuthArg*>(event.padding);
+		return res;
 
-		res = arg->result > 0 ? TRUE : FALSE;
+	auto arg = reinterpret_cast<SDL_UserAuthArg*>(event.padding);
 
-		free(*username);
-		free(*domain);
-		free(*password);
-		*username = arg->user;
-		*domain = arg->domain;
-		*password = arg->password;
-	}
+	res = arg->result > 0;
+
+	free(*username);
+	free(*domain);
+	free(*password);
+	*username = arg->user;
+	*domain = arg->domain;
+	*password = arg->password;
 
-fail:
-	free(title);
 	return res;
 }
 
@@ -190,24 +185,24 @@ BOOL sdl_choose_smartcard(freerdp* instance, SmartcardCertInfo** cert_list, DWOR
 		list.push_back(m.c_str());
 	}
 
-	SDL_Event event = { 0 };
+	SDL_Event event = {};
 	const char* title = "Select a logon smartcard certificate";
 	if (gateway)
 		title = "Select a gateway logon smartcard certificate";
 	if (!sdl_push_user_event(SDL_USEREVENT_SCARD_DIALOG, title, list.data(), count))
-		goto fail;
+		return res;
 
 	if (!sdl_wait_for_result(instance->context, SDL_USEREVENT_SCARD_RESULT, &event))
-		goto fail;
+		return res;
 
-	res = (event.user.code >= 0) ? TRUE : FALSE;
+	res = (event.user.code >= 0);
 	*choice = static_cast<DWORD>(event.user.code);
 
-fail:
 	return res;
 }
 
-SSIZE_T sdl_retry_dialog(freerdp* instance, const char* what, size_t current, void* userarg)
+SSIZE_T sdl_retry_dialog(freerdp* instance, const char* what, size_t current,
+                         [[maybe_unused]] void* userarg)
 {
 	WINPR_ASSERT(instance);
 	WINPR_ASSERT(instance->context);
@@ -216,9 +211,9 @@ SSIZE_T sdl_retry_dialog(freerdp* instance, const char* what, size_t current, vo
 	auto sdl = get_context(instance->context);
 	auto settings = instance->context->settings;
 	const size_t delay = freerdp_settings_get_uint32(settings, FreeRDP_TcpConnectTimeout);
-	std::lock_guard<CriticalSection> lock(sdl->critical);
+	std::scoped_lock lock(sdl->critical);
 	if (!sdl->connection_dialog)
-		return delay;
+		return WINPR_ASSERTING_INT_CAST(SSIZE_T, delay);
 
 	sdl->connection_dialog->setTitle("Retry connection to %s",
 	                                 freerdp_settings_get_server_name(instance->context->settings));
@@ -258,12 +253,13 @@ SSIZE_T sdl_retry_dialog(freerdp* instance, const char* what, size_t current, vo
 
 	sdl->connection_dialog->showInfo("[%s] retry %" PRIuz "/%" PRIuz ", delaying %" PRIuz
 	                                 "ms before next attempt",
-	                                 what, current, max, delay);
-	return delay;
+	                                 what, current + 1, max, delay);
+	return WINPR_ASSERTING_INT_CAST(SSIZE_T, delay);
 }
 
-BOOL sdl_present_gateway_message(freerdp* instance, UINT32 type, BOOL isDisplayMandatory,
-                                 BOOL isConsentMandatory, size_t length, const WCHAR* wmessage)
+BOOL sdl_present_gateway_message(freerdp* instance, [[maybe_unused]] UINT32 type,
+                                 BOOL isDisplayMandatory, BOOL isConsentMandatory, size_t length,
+                                 const WCHAR* wmessage)
 {
 	if (!isDisplayMandatory)
 		return TRUE;
@@ -283,7 +279,7 @@ BOOL sdl_present_gateway_message(freerdp* instance, UINT32 type, BOOL isDisplayM
 	const int rc = sdl_show_dialog(instance->context, title, message, flags);
 	free(title);
 	free(message);
-	return rc > 0 ? TRUE : FALSE;
+	return rc > 0;
 }
 
 int sdl_logon_error_info(freerdp* instance, UINT32 data, UINT32 type)
@@ -323,7 +319,7 @@ static DWORD sdl_show_ceritifcate_dialog(rdpContext* context, const char* title,
 	if (!sdl_push_user_event(SDL_USEREVENT_CERT_DIALOG, title, message))
 		return 0;
 
-	SDL_Event event = { 0 };
+	SDL_Event event = {};
 	if (!sdl_wait_for_result(context, SDL_USEREVENT_CERT_RESULT, &event))
 		return 0;
 	return static_cast<DWORD>(event.user.code);
@@ -619,9 +615,10 @@ BOOL sdl_auth_dialog_show(const SDL_UserAuthArg* args)
 
 BOOL sdl_scard_dialog_show(const char* title, Sint32 count, const char** list)
 {
+	const auto scount = WINPR_ASSERTING_INT_CAST(size_t, count);
 	std::vector<std::string> vlist;
-	vlist.reserve(count);
-	for (Sint32 x = 0; x < count; x++)
+	vlist.reserve(scount);
+	for (size_t x = 0; x < scount; x++)
 		vlist.emplace_back(list[x]);
 	SdlSelectList slist(title, vlist);
 	Sint32 value = slist.run();
