@@ -126,6 +126,7 @@ static const _SERIAL_IOCTL_NAME S_SERIAL_IOCTL_NAMES[] = {
 
 	{ IOCTL_USBPRINT_GET_1284_ID, "IOCTL_USBPRINT_GET_1284_ID" }
 };
+
 const char* _comm_serial_ioctl_name(ULONG number)
 {
 	for (size_t x = 0; x < ARRAYSIZE(S_SERIAL_IOCTL_NAMES); x++)
@@ -195,14 +196,18 @@ static BOOL CommInitialized(void)
 	return TRUE;
 }
 
-void CommLog_Print(DWORD level, ...)
+WINPR_ATTR_FORMAT_ARG(5, 6)
+void CommLog_PrintEx(DWORD level, const char* file, size_t line, const char* fkt,
+                     WINPR_FORMAT_ARG const char* fmt, ...)
 {
 	if (!CommInitialized())
 		return;
 
+	if (!WLog_IsLevelActive(sLog, level))
+		return;
 	va_list ap = { 0 };
-	va_start(ap, level);
-	WLog_PrintVA(sLog, level, ap);
+	va_start(ap, fmt);
+	WLog_PrintTextMessageVA(sLog, level, line, file, fkt, fmt, ap);
 	va_end(ap);
 }
 
@@ -501,8 +506,8 @@ BOOL GetCommState(HANDLE hFile, LPDCB lpDCB)
 	lpLocalDcb->fAbortOnError = (handflow.ControlHandShake & SERIAL_ERROR_ABORT) != 0;
 	/* lpLocalDcb->fDummy2 not used */
 	lpLocalDcb->wReserved = 0; /* must be zero */
-	lpLocalDcb->XonLim = handflow.XonLimit;
-	lpLocalDcb->XoffLim = handflow.XoffLimit;
+	lpLocalDcb->XonLim = WINPR_ASSERTING_INT_CAST(WORD, handflow.XonLimit);
+	lpLocalDcb->XoffLim = WINPR_ASSERTING_INT_CAST(WORD, handflow.XoffLimit);
 	SERIAL_LINE_CONTROL lineControl = { 0 };
 
 	if (!CommDeviceIoControl(pComm, IOCTL_SERIAL_GET_LINE_CONTROL, NULL, 0, &lineControl,
@@ -640,7 +645,7 @@ BOOL SetCommState(HANDLE hFile, LPDCB lpDCB)
 			break;
 
 		default:
-			CommLog_Print(WLOG_WARN, "Unexpected fDtrControl value: %" PRIu32 "\n",
+			CommLog_Print(WLOG_WARN, "Unexpected fDtrControl value: %" PRId32 "\n",
 			              lpDCB->fDtrControl);
 			return FALSE;
 	}
@@ -695,7 +700,7 @@ BOOL SetCommState(HANDLE hFile, LPDCB lpDCB)
 			break;
 
 		default:
-			CommLog_Print(WLOG_WARN, "Unexpected fRtsControl value: %" PRIu32 "\n",
+			CommLog_Print(WLOG_WARN, "Unexpected fRtsControl value: %" PRId32 "\n",
 			              lpDCB->fRtsControl);
 			return FALSE;
 	}
@@ -728,7 +733,7 @@ BOOL SetCommState(HANDLE hFile, LPDCB lpDCB)
 
 	if (lpDCB->fBinary)
 	{
-		upcomingTermios.c_lflag &= WINPR_ASSERTING_INT_CAST(tcflag_t, ~ICANON);
+		upcomingTermios.c_lflag &= (tcflag_t)~ICANON;
 	}
 	else
 	{
@@ -742,7 +747,7 @@ BOOL SetCommState(HANDLE hFile, LPDCB lpDCB)
 	}
 	else
 	{
-		upcomingTermios.c_iflag &= WINPR_ASSERTING_INT_CAST(tcflag_t, ~INPCK);
+		upcomingTermios.c_iflag &= (tcflag_t)~INPCK;
 	}
 
 	/* http://msdn.microsoft.com/en-us/library/windows/desktop/aa363423%28v=vs.85%29.aspx
@@ -756,7 +761,7 @@ BOOL SetCommState(HANDLE hFile, LPDCB lpDCB)
 	 * TCSANOW matches the best this definition
 	 */
 
-	if (_comm_ioctl_tcsetattr(pComm->fd, TCSANOW, &upcomingTermios) < 0)
+	if (comm_ioctl_tcsetattr(pComm->fd, TCSANOW, &upcomingTermios) < 0)
 	{
 		SetLastError(ERROR_IO_DEVICE);
 		return FALSE;
@@ -1384,22 +1389,7 @@ HANDLE CommCreateFileA(LPCSTR lpDeviceName, DWORD dwDesiredAccess, DWORD dwShare
 	pComm->serverSerialDriverId = SerialDriverUnknown;
 	InitializeCriticalSection(&pComm->EventsLock);
 
-#if defined(WINPR_HAVE_COMM_COUNTERS)
-	if (ioctl(pComm->fd, TIOCGICOUNT, &(pComm->counters)) < 0)
-	{
-		char ebuffer[256] = { 0 };
-		CommLog_Print(WLOG_WARN, "TIOCGICOUNT ioctl failed, errno=[%d] %s.", errno,
-		              winpr_strerror(errno, ebuffer, sizeof(ebuffer)));
-		CommLog_Print(WLOG_WARN, "could not read counters.");
-		/* could not initialize counters but keep on.
-		 *
-		 * Not all drivers, especially for USB to serial
-		 * adapters (e.g. those based on pl2303), does support
-		 * this call.
-		 */
-		ZeroMemory(&(pComm->counters), sizeof(struct serial_icounter_struct));
-	}
-#endif
+	(void)CommUpdateIOCount(pComm, TRUE);
 
 	/* The binary/raw mode is required for the redirection but
 	 * only flags that are not handle somewhere-else, except
@@ -1412,8 +1402,8 @@ HANDLE CommCreateFileA(LPCSTR lpDeviceName, DWORD dwDesiredAccess, DWORD dwShare
 		goto error_handle;
 	}
 
-	upcomingTermios.c_iflag &= WINPR_ASSERTING_INT_CAST(
-	    tcflag_t, ~(/*IGNBRK |*/ BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL /*| IXON*/));
+	upcomingTermios.c_iflag &=
+	    (tcflag_t) ~(/*IGNBRK |*/ BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL /*| IXON*/);
 	upcomingTermios.c_oflag = 0; /* <=> &= ~OPOST */
 	upcomingTermios.c_lflag = 0; /* <=> &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN); */
 	/* upcomingTermios.c_cflag &= ~(CSIZE | PARENB); */
@@ -1426,7 +1416,7 @@ HANDLE CommCreateFileA(LPCSTR lpDeviceName, DWORD dwDesiredAccess, DWORD dwShare
 	/* a few more settings required for the redirection */
 	upcomingTermios.c_cflag |= CLOCAL | CREAD;
 
-	if (_comm_ioctl_tcsetattr(pComm->fd, TCSANOW, &upcomingTermios) < 0)
+	if (comm_ioctl_tcsetattr(pComm->fd, TCSANOW, &upcomingTermios) < 0)
 	{
 		SetLastError(ERROR_IO_DEVICE);
 		goto error_handle;
@@ -1504,3 +1494,426 @@ int eventfd_write(int fd, eventfd_t value)
 }
 #endif
 #endif
+
+static const char* CommIoCtlToStr(unsigned long int io)
+{
+	switch (io)
+	{
+#if defined(WINPR_HAVE_SERIAL_SUPPORT)
+#if defined(TCGETS)
+		case TCGETS:
+			return "TCGETS";
+#endif
+#if defined(TCSETS)
+		case TCSETS:
+			return "TCSETS";
+#endif
+#if defined(TCSETSW)
+		case TCSETSW:
+			return "TCSETSW";
+#endif
+#if defined(TCSETSF)
+		case TCSETSF:
+			return "TCSETSF";
+#endif
+#if defined(TCGETA)
+		case TCGETA:
+			return "TCGETA";
+#endif
+#if defined(TCSETA)
+		case TCSETA:
+			return "TCSETA";
+#endif
+#if defined(TCSETAW)
+		case TCSETAW:
+			return "TCSETAW";
+#endif
+#if defined(TCSETAF)
+		case TCSETAF:
+			return "TCSETAF";
+#endif
+#if defined(TCSBRK)
+		case TCSBRK:
+			return "TCSBRK";
+#endif
+#if defined(TCXONC)
+		case TCXONC:
+			return "TCXONC";
+#endif
+#if defined(TCFLSH)
+		case TCFLSH:
+			return "TCFLSH";
+#endif
+#if defined(TIOCEXCL)
+		case TIOCEXCL:
+			return "TIOCEXCL";
+#endif
+#if defined(TIOCNXCL)
+		case TIOCNXCL:
+			return "TIOCNXCL";
+#endif
+#if defined(TIOCSCTTY)
+		case TIOCSCTTY:
+			return "TIOCSCTTY";
+#endif
+#if defined(TIOCGPGRP)
+		case TIOCGPGRP:
+			return "TIOCGPGRP";
+#endif
+#if defined(TIOCSPGRP)
+		case TIOCSPGRP:
+			return "TIOCSPGRP";
+#endif
+#if defined(TIOCOUTQ)
+		case TIOCOUTQ:
+			return "TIOCOUTQ";
+#endif
+#if defined(TIOCSTI)
+		case TIOCSTI:
+			return "TIOCSTI";
+#endif
+#if defined(TIOCGWINSZ)
+		case TIOCGWINSZ:
+			return "TIOCGWINSZ";
+#endif
+#if defined(TIOCSWINSZ)
+		case TIOCSWINSZ:
+			return "TIOCSWINSZ";
+#endif
+#if defined(TIOCMGET)
+		case TIOCMGET:
+			return "TIOCMGET";
+#endif
+#if defined(TIOCMBIS)
+		case TIOCMBIS:
+			return "TIOCMBIS";
+#endif
+#if defined(TIOCMBIC)
+		case TIOCMBIC:
+			return "TIOCMBIC";
+#endif
+#if defined(TIOCMSET)
+		case TIOCMSET:
+			return "TIOCMSET";
+#endif
+#if defined(TIOCGSOFTCAR)
+		case TIOCGSOFTCAR:
+			return "TIOCGSOFTCAR";
+#endif
+#if defined(TIOCSSOFTCAR)
+		case TIOCSSOFTCAR:
+			return "TIOCSSOFTCAR";
+#endif
+#if defined(FIONREAD)
+		case FIONREAD:
+			return "FIONREAD/TIOCINQ";
+#endif
+#if defined(TIOCLINUX)
+		case TIOCLINUX:
+			return "TIOCLINUX";
+#endif
+#if defined(TIOCCONS)
+		case TIOCCONS:
+			return "TIOCCONS";
+#endif
+#if defined(TIOCGSERIAL)
+		case TIOCGSERIAL:
+			return "TIOCGSERIAL";
+#endif
+#if defined(TIOCSSERIAL)
+		case TIOCSSERIAL:
+			return "TIOCSSERIAL";
+#endif
+#if defined(TIOCPKT)
+		case TIOCPKT:
+			return "TIOCPKT";
+#endif
+#if defined(FIONBIO)
+		case FIONBIO:
+			return "FIONBIO";
+#endif
+#if defined(TIOCNOTTY)
+		case TIOCNOTTY:
+			return "TIOCNOTTY";
+#endif
+#if defined(TIOCSETD)
+		case TIOCSETD:
+			return "TIOCSETD";
+#endif
+#if defined(TIOCGETD)
+		case TIOCGETD:
+			return "TIOCGETD";
+#endif
+#if defined(TCSBRKP)
+		case TCSBRKP:
+			return "TCSBRKP";
+#endif
+#if defined(TIOCSBRK)
+		case TIOCSBRK:
+			return "TIOCSBRK";
+#endif
+#if defined(TIOCCBRK)
+		case TIOCCBRK:
+			return "TIOCCBRK";
+#endif
+#if defined(TIOCGSID)
+		case TIOCGSID:
+			return "TIOCGSID";
+#endif
+#if defined(TIOCGRS485)
+		case TIOCGRS485:
+			return "TIOCGRS485";
+#endif
+#if defined(TIOCSRS485)
+		case TIOCSRS485:
+			return "TIOCSRS485";
+#endif
+#if defined(TIOCSPTLCK)
+		case TIOCSPTLCK:
+			return "TIOCSPTLCK";
+#endif
+#if defined(TCGETX)
+		case TCGETX:
+			return "TCGETX";
+#endif
+#if defined(TCSETX)
+		case TCSETX:
+			return "TCSETX";
+#endif
+#if defined(TCSETXF)
+		case TCSETXF:
+			return "TCSETXF";
+#endif
+#if defined(TCSETXW)
+		case TCSETXW:
+			return "TCSETXW";
+#endif
+#if defined(TIOCSIG)
+		case TIOCSIG:
+			return "TIOCSIG";
+#endif
+#if defined(TIOCVHANGUP)
+		case TIOCVHANGUP:
+			return "TIOCVHANGUP";
+#endif
+#if defined(TIOCGPTPEER)
+		case TIOCGPTPEER:
+			return "TIOCGPTPEER";
+#endif
+#if defined(FIONCLEX)
+		case FIONCLEX:
+			return "FIONCLEX";
+#endif
+#if defined(FIOCLEX)
+		case FIOCLEX:
+			return "FIOCLEX";
+#endif
+#if defined(FIOASYNC)
+		case FIOASYNC:
+			return "FIOASYNC";
+#endif
+#if defined(TIOCSERCONFIG)
+		case TIOCSERCONFIG:
+			return "TIOCSERCONFIG";
+#endif
+#if defined(TIOCSERGWILD)
+		case TIOCSERGWILD:
+			return "TIOCSERGWILD";
+#endif
+#if defined(TIOCSERSWILD)
+		case TIOCSERSWILD:
+			return "TIOCSERSWILD";
+#endif
+#if defined(TIOCGLCKTRMIOS)
+		case TIOCGLCKTRMIOS:
+			return "TIOCGLCKTRMIOS";
+#endif
+#if defined(TIOCSLCKTRMIOS)
+		case TIOCSLCKTRMIOS:
+			return "TIOCSLCKTRMIOS";
+#endif
+#if defined(TIOCSERGSTRUCT)
+		case TIOCSERGSTRUCT:
+			return "TIOCSERGSTRUCT";
+#endif
+#if defined(TIOCSERGETLSR)
+		case TIOCSERGETLSR:
+			return "TIOCSERGETLSR";
+#endif
+#if defined(TIOCSERGETMULTI)
+		case TIOCSERGETMULTI:
+			return "TIOCSERGETMULTI";
+#endif
+#if defined(TIOCSERSETMULTI)
+		case TIOCSERSETMULTI:
+			return "TIOCSERSETMULTI";
+#endif
+#if defined(TIOCMIWAIT)
+		case TIOCMIWAIT:
+			return "TIOCMIWAIT";
+#endif
+#if defined(TIOCGICOUNT)
+		case TIOCGICOUNT:
+			return "TIOCGICOUNT";
+#endif
+#if defined(FIOQSIZE)
+		case FIOQSIZE:
+			return "FIOQSIZE";
+#endif
+#if defined(TIOCPKT_DATA)
+		case TIOCPKT_DATA:
+			return "TIOCPKT_DATA";
+#endif
+#if defined(TIOCPKT_FLUSHWRITE)
+		case TIOCPKT_FLUSHWRITE:
+			return "TIOCPKT_FLUSHWRITE";
+#endif
+#if defined(TIOCPKT_STOP)
+		case TIOCPKT_STOP:
+			return "TIOCPKT_STOP";
+#endif
+#if defined(TIOCPKT_START)
+		case TIOCPKT_START:
+			return "TIOCPKT_START";
+#endif
+#if defined(TIOCPKT_NOSTOP)
+		case TIOCPKT_NOSTOP:
+			return "TIOCPKT_NOSTOP";
+#endif
+#if defined(TIOCPKT_DOSTOP)
+		case TIOCPKT_DOSTOP:
+			return "TIOCPKT_DOSTOP";
+#endif
+#if defined(TIOCPKT_IOCTL)
+		case TIOCPKT_IOCTL:
+			return "TIOCPKT_IOCTL";
+#endif
+#endif
+		default:
+			return "UNKNOWN";
+	}
+}
+
+static BOOL CommStatusErrorEx(WINPR_COMM* pComm, unsigned long int ctl, const char* file,
+                              const char* fkt, size_t line)
+{
+	WINPR_ASSERT(pComm);
+	BOOL rc = pComm->permissive ? TRUE : FALSE;
+	const DWORD level = rc ? WLOG_DEBUG : WLOG_WARN;
+	char ebuffer[256] = { 0 };
+	const char* str = CommIoCtlToStr(ctl);
+
+	if (CommInitialized())
+	{
+		if (WLog_IsLevelActive(sLog, level))
+		{
+			WLog_PrintTextMessage(sLog, level, line, file, fkt,
+			                      "%s [0x%08lx] ioctl failed, errno=[%d] %s.", str, ctl, errno,
+			                      winpr_strerror(errno, ebuffer, sizeof(ebuffer)));
+		}
+	}
+
+	if (!rc)
+		SetLastError(ERROR_IO_DEVICE);
+
+	return rc;
+}
+
+BOOL CommIoCtl_int(WINPR_COMM* pComm, unsigned long int ctl, void* data, const char* file,
+                   const char* fkt, size_t line)
+{
+	if (ioctl(pComm->fd, ctl, data) < 0)
+	{
+		if (!CommStatusErrorEx(pComm, ctl, file, fkt, line))
+			return FALSE;
+	}
+	return TRUE;
+}
+
+BOOL CommUpdateIOCount(WINPR_ATTR_UNUSED HANDLE handle, WINPR_ATTR_UNUSED BOOL checkSupportStatus)
+{
+	WINPR_COMM* pComm = (WINPR_COMM*)handle;
+	WINPR_ASSERT(pComm);
+
+#if defined(WINPR_HAVE_COMM_COUNTERS)
+	ZeroMemory(&(pComm->counters), sizeof(struct serial_icounter_struct));
+	if (pComm->TIOCGICOUNTSupported || checkSupportStatus)
+	{
+		const int rc = ioctl(pComm->fd, TIOCGICOUNT, &(pComm->counters));
+		if (checkSupportStatus)
+			pComm->TIOCGICOUNTSupported = rc >= 0;
+		else if (rc < 0)
+		{
+			if (!CommStatusErrorEx(pComm, TIOCGICOUNT, __FILE__, __func__, __LINE__))
+				return FALSE;
+		}
+	}
+#endif
+	return TRUE;
+}
+
+static const char* CommSerialEvFlagString(ULONG flag)
+{
+	switch (flag)
+	{
+		case SERIAL_EV_RXCHAR:
+			return "SERIAL_EV_RXCHAR";
+		case SERIAL_EV_RXFLAG:
+			return "SERIAL_EV_RXFLAG";
+		case SERIAL_EV_TXEMPTY:
+			return "SERIAL_EV_TXEMPTY";
+		case SERIAL_EV_CTS:
+			return "SERIAL_EV_CTS ";
+		case SERIAL_EV_DSR:
+			return "SERIAL_EV_DSR ";
+		case SERIAL_EV_RLSD:
+			return "SERIAL_EV_RLSD";
+		case SERIAL_EV_BREAK:
+			return "SERIAL_EV_BREAK";
+		case SERIAL_EV_ERR:
+			return "SERIAL_EV_ERR ";
+		case SERIAL_EV_RING:
+			return "SERIAL_EV_RING";
+		case SERIAL_EV_PERR:
+			return "SERIAL_EV_PERR";
+		case SERIAL_EV_RX80FULL:
+			return "SERIAL_EV_RX80FULL";
+		case SERIAL_EV_EVENT1:
+			return "SERIAL_EV_EVENT1";
+		case SERIAL_EV_EVENT2:
+			return "SERIAL_EV_EVENT2";
+		case SERIAL_EV_WINPR_WAITING:
+			return "SERIAL_EV_WINPR_WAITING";
+		case SERIAL_EV_WINPR_STOP:
+			return "SERIAL_EV_WINPR_STOP";
+		default:
+			return "SERIAL_EV_UNKNOWN";
+	}
+}
+
+const char* CommSerialEvString(ULONG status, char* buffer, size_t size)
+{
+	const ULONG flags[] = { SERIAL_EV_RXCHAR, SERIAL_EV_RXFLAG,        SERIAL_EV_TXEMPTY,
+		                    SERIAL_EV_CTS,    SERIAL_EV_DSR,           SERIAL_EV_RLSD,
+		                    SERIAL_EV_BREAK,  SERIAL_EV_ERR,           SERIAL_EV_RING,
+		                    SERIAL_EV_PERR,   SERIAL_EV_RX80FULL,      SERIAL_EV_EVENT1,
+		                    SERIAL_EV_EVENT2, SERIAL_EV_WINPR_WAITING, SERIAL_EV_WINPR_STOP };
+
+	winpr_str_append("{", buffer, size, "");
+
+	const char* sep = "";
+	for (size_t x = 0; x < ARRAYSIZE(flags); x++)
+	{
+		const ULONG flag = flags[x];
+		if (status & flag)
+		{
+			winpr_str_append(CommSerialEvFlagString(flag), buffer, size, sep);
+			sep = "|";
+		}
+	}
+
+	char number[32] = { 0 };
+	(void)_snprintf(number, sizeof(number), "}[0x%08" PRIx32 "]", status);
+	winpr_str_append(number, buffer, size, "");
+	return buffer;
+}
